/* Copyright (C) 2020 Google, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "panvk_tracepoints.h"


#define __NEEDS_TRACE_PRIV
#include "util/u_debug.h"
#include "util/perf/u_trace_priv.h"


/*
 * begin_cmdbuf
 */
#define __print_begin_cmdbuf NULL
#define __print_json_begin_cmdbuf NULL
static const struct u_tracepoint __tp_begin_cmdbuf = {
    "begin_cmdbuf",
    ALIGN_POT(sizeof(struct trace_begin_cmdbuf), 8),   /* keep size 64b aligned */
    0
    ,
    0,
    0,
    __print_begin_cmdbuf,
    __print_json_begin_cmdbuf,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, uint16_t, const void *, const void *, const void *))panvk_utrace_perfetto_begin_cmdbuf,
#endif
};
void __trace_begin_cmdbuf(
     struct u_trace *ut
   , enum u_trace_type enabled_traces
   , void *cs
) {
   struct trace_begin_cmdbuf entry;
   UNUSED struct trace_begin_cmdbuf *__entry =
      enabled_traces & U_TRACE_TYPE_REQUIRE_QUEUING ?
      (struct trace_begin_cmdbuf *)u_trace_appendv(ut, cs, &__tp_begin_cmdbuf,
                                                    0
                                                    ,
                                                    0, NULL, NULL
                                                    ) :
      &entry;
}

/*
 * end_cmdbuf
 */
static void __print_end_cmdbuf(FILE *out, const void *arg, const void *indirect) {
   const struct trace_end_cmdbuf *__entry =
      (const struct trace_end_cmdbuf *)arg;
   fprintf(out, ""
      "flags=0x%x, "
         "\n"
   ,__entry->flags
   );
}

static void __print_json_end_cmdbuf(FILE *out, const void *arg, const void *indirect) {
   const struct trace_end_cmdbuf *__entry =
      (const struct trace_end_cmdbuf *)arg;
   fprintf(out, ""
      "\"flags\": \"0x%x\""
   ,__entry->flags
   );
}

static const struct u_tracepoint __tp_end_cmdbuf = {
    "end_cmdbuf",
    ALIGN_POT(sizeof(struct trace_end_cmdbuf), 8),   /* keep size 64b aligned */
    0
    ,
    0,
    1,
    __print_end_cmdbuf,
    __print_json_end_cmdbuf,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, uint16_t, const void *, const void *, const void *))panvk_utrace_perfetto_end_cmdbuf,
#endif
};
void __trace_end_cmdbuf(
     struct u_trace *ut
   , enum u_trace_type enabled_traces
   , void *cs
   , VkCommandBufferUsageFlags flags
) {
   struct trace_end_cmdbuf entry;
   UNUSED struct trace_end_cmdbuf *__entry =
      enabled_traces & U_TRACE_TYPE_REQUIRE_QUEUING ?
      (struct trace_end_cmdbuf *)u_trace_appendv(ut, cs, &__tp_end_cmdbuf,
                                                    0
                                                    ,
                                                    0, NULL, NULL
                                                    ) :
      &entry;
   __entry->flags = flags;
}

