/* Copyright (C) 2020 Google, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */


#ifndef _PANVK_TRACEPOINTS_H
#define _PANVK_TRACEPOINTS_H

#include "vulkan/vulkan_core.h"

#include "util/perf/u_trace.h"

#ifdef __cplusplus
extern "C" {
#endif

struct panvk_device;



/*
 * begin_cmdbuf
 */
struct trace_begin_cmdbuf {
#ifdef __cplusplus
   /* avoid warnings about empty struct size mis-match in C vs C++..
    * the size mis-match is harmless because (a) nothing will deref
    * the empty struct, and (b) the code that cares about allocating
    * sizeof(struct trace_begin_cmdbuf) (and wants this to be zero
    * if there is no payload) is C
    */
   uint8_t dummy;
#endif
};
#ifdef HAVE_PERFETTO
void panvk_utrace_perfetto_begin_cmdbuf(
   struct panvk_device *dev,
   uint64_t ts_ns,
   uint16_t tp_idx,
   const void *flush_data,
   const struct trace_begin_cmdbuf *payload,
   const void *indirect_data);
#endif
void __trace_begin_cmdbuf(
       struct u_trace *ut
     , enum u_trace_type enabled_traces
     , void *cs
);
static ALWAYS_INLINE void trace_begin_cmdbuf(
     struct u_trace *ut
   , void *cs
) {
   enum u_trace_type enabled_traces = p_atomic_read_relaxed(&ut->utctx->enabled_traces);
   if (!unlikely(enabled_traces != 0 &&
                 true))
      return;
   __trace_begin_cmdbuf(
        ut
      , enabled_traces
      , cs
   );
}

/*
 * end_cmdbuf
 */
struct trace_end_cmdbuf {
   VkCommandBufferUsageFlags flags;
};
#ifdef HAVE_PERFETTO
void panvk_utrace_perfetto_end_cmdbuf(
   struct panvk_device *dev,
   uint64_t ts_ns,
   uint16_t tp_idx,
   const void *flush_data,
   const struct trace_end_cmdbuf *payload,
   const void *indirect_data);
#endif
void __trace_end_cmdbuf(
       struct u_trace *ut
     , enum u_trace_type enabled_traces
     , void *cs
     , VkCommandBufferUsageFlags flags
);
static ALWAYS_INLINE void trace_end_cmdbuf(
     struct u_trace *ut
   , void *cs
   , VkCommandBufferUsageFlags flags
) {
   enum u_trace_type enabled_traces = p_atomic_read_relaxed(&ut->utctx->enabled_traces);
   if (!unlikely(enabled_traces != 0 &&
                 true))
      return;
   __trace_end_cmdbuf(
        ut
      , enabled_traces
      , cs
      , flags
   );
}

#ifdef __cplusplus
}
#endif

#endif /* _PANVK_TRACEPOINTS_H */
